from __future__ import with_statement
import os
import asyncio
from threading import Thread

from alembic import context
from pscr_common.log import setup_logging
from sqlalchemy import engine_from_config, pool
from sqlalchemy.ext.asyncio import AsyncEngine

from feed_manager.utils.config import Config


# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
from feed_manager.utils.db import get_engine

config = context.config

setup_logging(
    level=Config.create().get("log", "level"),
    svc_version=os.environ.get("CONTAINERVERSION", "UNKNOWN"),
    env_id=os.environ.get("ENV_ID", "UNKNOWN"),
    svc_id="TODO",
)

# add your model's MetaData object here
# for 'autogenerate' support
from feed_manager.sql.models import Base

target_metadata = Base.metadata

# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.


def run_migrations_offline():
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    url = config.get_main_option("sqlalchemy.url")
    context.configure(url=url, target_metadata=target_metadata, literal_binds=True)

    with context.begin_transaction():
        context.run_migrations()


def do_run_migrations(connection):
    context.configure(connection=connection, target_metadata=target_metadata)
    context.run_migrations()


async def run_migrations_online():
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """
    db_conf = config.attributes.get("config", None)
    engine = get_engine(db_conf)
    async with engine.begin() as connection:
        await connection.run_sync(do_run_migrations)


if context.is_offline_mode():
    run_migrations_offline()
else:

    def start_background_loop(l: asyncio.AbstractEventLoop) -> None:
        asyncio.set_event_loop(l)
        l.run_forever()

    loop = asyncio.new_event_loop()
    t = Thread(target=start_background_loop, args=(loop,), daemon=True)
    t.start()
    task = asyncio.run_coroutine_threadsafe(run_migrations_online(), loop)
    task.result()
    loop.stop()
