"""Add org watchlist state table

Revision ID: 75b1d2d16b74
Revises: 4f8ecb70c4b4
Create Date: 2019-02-12 09:01:23.045077

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy import text


# revision identifiers, used by Alembic.
from pscr_common.utils import get_utcnow_with_tzinfo, EPOCH
from sqlalchemy import null
from sqlalchemy.dialects.postgresql import insert

from feed_manager.sql.models import OrganizationWatchlistState

revision = "75b1d2d16b74"
down_revision = "4f8ecb70c4b4"
branch_labels = None
depends_on = None


def upgrade():
    op.create_table(
        "organization_watchlist_states",
        sa.Column("org_id", sa.String(), nullable=False),
        sa.Column("dirty_timestamp", sa.DateTime(timezone=True), nullable=True),
        sa.PrimaryKeyConstraint("org_id"),
    )

    conn = op.get_bind()
    for result in conn.execute(
        text("SELECT DISTINCT ON (org_id) org_id FROM watchlists")
    ):
        org_id = result["org_id"]

        insert_stmt = (
            insert(OrganizationWatchlistState)
            .values(org_id=org_id, dirty_timestamp=EPOCH)
            .returning(OrganizationWatchlistState.org_id)
        )

        upsert_stmt = insert_stmt.on_conflict_do_update(
            index_elements=[OrganizationWatchlistState.org_id],
            set_={"dirty_timestamp": insert_stmt.excluded.dirty_timestamp},
            where=OrganizationWatchlistState.dirty_timestamp == null(),
        )

        conn.execute(upsert_stmt)


def downgrade():  # pragma: no cover
    op.drop_table("organization_watchlist_states")
